@@ -11,7 +11,10 @@ import sys
 import warnings
 from pathlib import Path
 
-from fusesoc import __version__
+try:
+    from fusesoc.version import version as __version__
+except ImportError:
+    __version__ = "unknown"
 
 # Check if this is run from a local installation
 fusesocdir = os.path.abspath(
@@ -100,12 +103,15 @@ def list_paths(fs, args):
 def add_library(fs, args):
     sync_uri = vars(args)["sync-uri"]
 
+    name = args.name or os.path.basename(sync_uri.rstrip("/"))
+
+    # Check where to store the library
     if args.location:
         location = args.location
     elif vars(args).get("global", False):
-        location = os.path.join(fs.config.library_root, args.name)
+        location = os.path.join(fs.config.library_root, name)
     else:
-        location = os.path.join("fusesoc_libraries", args.name)
+        location = os.path.join("fusesoc_libraries", name)
 
     sync_type = vars(args).get("sync-type")
     sync_version = vars(args).get("sync-version")
@@ -126,7 +132,7 @@ def add_library(fs, args):
         location = os.path.abspath(sync_uri)
 
     auto_sync = not args.no_auto_sync
-    library = Library(args.name, location, sync_type, sync_uri, sync_version, auto_sync)
+    library = Library(name, location, sync_type, sync_uri, sync_version, auto_sync)
 
     if args.config:
         config = Config(args.config)
@@ -200,7 +206,7 @@ def list_cores(fs, args):
 
 
 def list_tools(fs, args):
-    from edalize.edatool import walk_tool_packages
+    from edalize.edatool import get_edatool, walk_tool_packages
 
     _tp = list(walk_tool_packages())
     maxlen = max(map(len, _tp))
@@ -304,11 +310,18 @@ def run(fs, args):
     except SyntaxError as e:
         logger.error(str(e))
         exit(1)
+    except RuntimeError as e:
+        logger.error(str(e))
+        exit(1)
 
     # Unconditionally clean out the work root on fresh builds
     # if we use the old tool API
     if do_configure and not core.get_flow(flags):
-        prepare_work_root(fs.get_work_root(core, flags))
+        try:
+            prepare_work_root(fs.get_work_root(core, flags))
+        except RuntimeError as e:
+            logger.error(e)
+            exit(1)
 
     # Frontend/backend separation
 
@@ -503,7 +516,11 @@ def get_parser():
     parser_library_add = library_subparsers.add_parser(
         "add", help="Add new library to fusesoc.conf"
     )
-    parser_library_add.add_argument("name", help="A friendly name  for the library")
+    parser_library_add.add_argument(
+        "name",
+        nargs="?",
+        help="A friendly name  for the library. Defaults to last part of sync-uri if not specified.",
+    )
     parser_library_add.add_argument(
         "sync-uri", help="The URI source for the library (can be a file system path)"
     )
@@ -514,7 +531,7 @@ def get_parser():
     )
     parser_library_add.add_argument(
         "--location",
-        help="The location to store the library into (defaults to $XDG_DATA_HOME/[name])",
+        help="The location to store the library into (defaults to fusesoc_libraries/[name] or $XDG_DATA_HOME/[name] when --global is set)",
     )
     parser_library_add.add_argument(
         "--sync-type",
@@ -564,6 +581,12 @@ def get_parser():
         "--work-root",
         help="Output directory for build. VLNV will not be appended (overrides build-root)",
     )
+    parser_run.add_argument(
+        "--filter",
+        help="Add filter. Can be specified multiple times to add multiple filters",
+        action="append",
+        default=[],
+    )
     parser_run.add_argument("--setup", action="store_true", help="Execute setup stage")
     parser_run.add_argument("--build", action="store_true", help="Execute build stage")
     parser_run.add_argument("--run", action="store_true", help="Execute run stage")
@@ -649,13 +672,15 @@ def args_to_config(args, config):
     if hasattr(args, "system_name") and args.system_name and len(args.system_name) > 0:
         setattr(config, "args_system_name", args.system_name)
 
+    if hasattr(args, "filter"):
+        config.args_filters = args.filter
+
 
 def fusesoc(args):
     Fusesoc.init_logging(args.verbose, args.monochrome, args.log_file)
 
     config = Config(args.config)
     args_to_config(args, config)
-
     fs = Fusesoc(config)
 
     # Run the function
